use strict;
use warnings;
use Test::More tests => 5;

use_ok 'App::clad';
use_ok 'Clustericious::Admin';
use_ok 'Clustericious::Admin::Server';
use_ok 'Clustericious::Admin::RemoteHandler';
use_ok 'Clustericious::Admin::Dump';
